@@ -20,13 +20,15 @@
    Author: Bastien Nocera <hadess@hadess.net>
  */
 
-#ifndef TOTEM_SUBLANG_H
-#define TOTEM_SUBLANG_H
+#ifndef TOTEM_MENU_H
+#define TOTEM_MENU_H
 
 #include "totem.h"
 
 G_BEGIN_DECLS
 
+void totem_ui_manager_setup (Totem *totem);
+
 void totem_sublang_update (Totem *totem);
 void totem_sublang_exit (Totem *totem);
 
@@ -37,4 +39,4 @@ void totem_action_add_recent (Totem *totem, const char *filename);
 
 G_END_DECLS
 
-#endif /* TOTEM_SUBLANG_H */
+#endif /* TOTEM_MENU_H */
