@@ -21,9 +21,6 @@
 
 #include "config.h"
 
-//FIXME damn build system!
-#include "totem-interface.c"
-
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 #include <stdlib.h>
@@ -44,9 +41,11 @@
 #include "bacon-video-widget.h"
 #include "totem-interface.h"
 #include "totem-mozilla-options.h"
+#include "totem-statusbar.h"
 #include "bacon-volume.h"
 #include "video-utils.h"
 
+GtkWidget *totem_statusbar_create (void);
 GtkWidget *totem_volume_create (void);
 
 #define IS_FD_STREAM (strcmp(emb->filename, "fd://0") == 0)
@@ -78,6 +77,7 @@ typedef struct _TotemEmbedded {
 	GtkWidget *window;
 	GladeXML *menuxml, *xml;
 	GtkWidget *about;
+	TotemStatusbar *statusbar;
 	int width, height;
 	const char *orig_filename;
 	const char *mimetype;
@@ -174,7 +174,7 @@ totem_embedded_emit_stop_sending_data (TotemEmbedded *emb)
 static void
 totem_embedded_set_state (TotemEmbedded *emb, TotemStates state)
 {
-	const char *id = NULL;
+	char *id = NULL;
 	GtkWidget *image;
 	GdkCursor *cursor;
 
@@ -182,26 +182,33 @@ totem_embedded_set_state (TotemEmbedded *emb, TotemStates state)
 		return;
 
 	cursor = NULL;
+	image = glade_xml_get_widget (emb->xml, "emb_pp_button_image");
 
 	switch (state) {
 	case STATE_STOPPED:
-		id = GTK_STOCK_MEDIA_PLAY;
 		if (emb->href != NULL)
 			cursor = emb->cursor;
+		id = g_strdup_printf ("gtk-media-play-%s",
+				gtk_widget_get_direction (image) ? "ltr" : "rtl");
+		totem_statusbar_set_text (emb->statusbar, _("Stopped"));
+		totem_statusbar_set_time_and_length (emb->statusbar, 0, 0);
 		break;
 	case STATE_PAUSED:
-		id = GTK_STOCK_MEDIA_PLAY;
+		id = g_strdup_printf ("gtk-media-play-%s",
+				gtk_widget_get_direction (image) ? "ltr" : "rtl");
+		totem_statusbar_set_text (emb->statusbar, _("Paused"));
 		break;
 	case STATE_PLAYING:
-		id = GTK_STOCK_MEDIA_PAUSE;
+		id = g_strdup ("gtk-media-pause");
+		totem_statusbar_set_text (emb->statusbar, _("Playing"));
 		break;
 	default:
 		break;
 	}
 
-	image = glade_xml_get_widget (emb->xml, "emb_pp_button_image");
-	gtk_image_set_from_stock (GTK_IMAGE (image), id, GTK_ICON_SIZE_MENU);
-	if (emb->hidden == FALSE && cursor)
+	gtk_image_set_from_icon_name (GTK_IMAGE (image), id, GTK_ICON_SIZE_MENU);
+	g_free (id);
+	if (emb->hidden == FALSE && cursor != NULL)
 		gdk_window_set_cursor (GTK_WIDGET (emb->bvw)->window, cursor);
 
 	emb->state = state;
@@ -252,7 +259,7 @@ totem_embedded_open (TotemEmbedded *emb)
 		totem_embedded_set_pp_state (emb, TRUE);
 	}
 
-	if (retval == FALSE || IS_FD_STREAM) {
+	if (IS_FD_STREAM && emb->num_items > 1) {
 		totem_embedded_set_menu (emb, FALSE);
 	} else {
 		totem_embedded_set_menu (emb, TRUE);
@@ -319,7 +326,14 @@ totem_embedded_set_menu (TotemEmbedded *emb, gboolean enable)
 	if (enable == FALSE)
 		return;
 
-	emb->app = gnome_vfs_mime_get_default_application_for_uri (emb->filename, emb->mimetype);
+	if (IS_FD_STREAM) {
+		emb->app = gnome_vfs_mime_get_default_application_for_uri
+			(emb->orig_filename, emb->mimetype);
+	} else {
+		emb->app = gnome_vfs_mime_get_default_application_for_uri
+			(emb->filename, emb->mimetype);
+	}
+
 	if (emb->app == NULL)
 		return;
 
@@ -347,7 +361,14 @@ on_open1_activate (GtkButton *button, TotemEmbedded *emb)
 
 	g_return_if_fail (emb->app != NULL);
 
-	l = g_list_prepend (l, emb->filename);
+	if (IS_FD_STREAM) {
+		l = g_list_prepend (l, (gpointer) emb->orig_filename);
+	} else if (emb->href != NULL) {
+		l = g_list_prepend (l, emb->href);
+	} else {
+		l = g_list_prepend (l, emb->filename);
+	}
+
 	if (gnome_vfs_mime_application_launch (emb->app, l) == GNOME_VFS_OK) {
 		totem_embedded_stop (emb, NULL);
 	}
@@ -405,14 +426,23 @@ static void
 on_copy_location1_activate (GtkButton *button, TotemEmbedded *emb)
 {
 	GtkClipboard *clip;
+	const char *filename;
+
+	if (IS_FD_STREAM) {
+		filename = emb->orig_filename;
+	} else if (emb->href != NULL) {
+		filename = emb->href;
+	} else {
+		filename = emb->filename;
+	}
 
 	/* Set both the middle-click and the super-paste buffers */
 	clip = gtk_clipboard_get_for_display
 		(gdk_display_get_default(), GDK_SELECTION_CLIPBOARD);
-	gtk_clipboard_set_text (clip, emb->filename, -1);
+	gtk_clipboard_set_text (clip, filename, -1);
 	clip = gtk_clipboard_get_for_display
 		(gdk_display_get_default(), GDK_SELECTION_PRIMARY);
-	gtk_clipboard_set_text (clip, emb->filename, -1);
+	gtk_clipboard_set_text (clip, filename, -1);
 }
 
 static void
@@ -555,6 +585,22 @@ on_eos_event (GtkWidget *bvw, TotemEmbedded *emb)
 	}
 }
 
+static void
+on_error_event (BaconVideoWidget *bvw, char *message,
+                gboolean playback_stopped, gboolean fatal, TotemEmbedded *emb)
+{
+	if (playback_stopped)
+		totem_embedded_set_state (emb, STATE_STOPPED);
+
+	if (fatal == FALSE) {
+		totem_interface_error (_("An error occurred"), message,
+		                       GTK_WINDOW (emb->window));
+	} else {
+		totem_embedded_error_and_exit (_("An error occurred"),
+				message, emb);
+	}
+}
+
 static void
 cb_vol (GtkWidget *val, TotemEmbedded *emb)
 {
@@ -600,6 +646,14 @@ on_tick (GtkWidget *bvw,
 		if (emb->seeking == FALSE)
 			gtk_adjustment_set_value (emb->seekadj,
 					current_position * 65535);
+		if (stream_length == 0) {
+			totem_statusbar_set_time_and_length (emb->statusbar,
+					(int) (current_time / 1000), -1);
+		} else {
+			totem_statusbar_set_time_and_length (emb->statusbar,
+					(int) (current_time / 1000),
+					(int) (stream_length / 1000));
+		}
 	}
 }
 
@@ -660,6 +714,8 @@ totem_embedded_add_children (TotemEmbedded *emb)
 			G_CALLBACK (on_got_redirect), emb);
 	g_signal_connect (G_OBJECT (emb->bvw), "eos",
 			G_CALLBACK (on_eos_event), emb);
+	g_signal_connect (G_OBJECT (emb->bvw), "error",
+			G_CALLBACK (on_error_event), emb);
 	g_signal_connect (G_OBJECT(emb->bvw), "button-press-event",
 			G_CALLBACK (on_video_button_press_event), emb);
 	g_signal_connect (G_OBJECT(emb->bvw), "tick",
@@ -696,6 +752,9 @@ totem_embedded_add_children (TotemEmbedded *emb)
 	g_signal_connect (G_OBJECT (vbut), "scroll_event",
 			G_CALLBACK (on_volume_scroll_event), emb);
 
+	emb->statusbar = TOTEM_STATUSBAR (glade_xml_get_widget
+			(emb->xml, "emb_statusbar"));
+
 	gtk_widget_realize (emb->window);
 	gtk_widget_set_size_request (emb->window, emb->width, emb->height);
 
@@ -803,18 +862,6 @@ static void embedded (GtkPlug *plug, TotemEmbedded *emb)
 	emb->embedded_done = TRUE;
 }
 
-GtkWidget *
-totem_volume_create (void)
-{
-	GtkWidget *widget;
-
-	widget = bacon_volume_button_new (GTK_ICON_SIZE_SMALL_TOOLBAR,
-					  0, 100, -1);
-	gtk_widget_show (widget);
-
-	return widget;
-}
-
 static int arg_width = -1;
 static int arg_height = -1;
 static char *arg_url = NULL;
@@ -822,13 +869,39 @@ static char *arg_href = NULL;
 static char *arg_target = NULL;
 static char *arg_mime_type = NULL;
 static char **arg_remaining = NULL;
-static Window xid;
+static Window xid = 0;
 static gboolean arg_no_controls = FALSE;
+static gboolean arg_statusbar = FALSE;
 static gboolean arg_hidden = FALSE;
 static gboolean arg_is_playlist = FALSE;
 static gboolean arg_repeat = FALSE;
 static gboolean arg_no_autostart = FALSE;
 
+GtkWidget *
+totem_volume_create (void)
+{
+	GtkWidget *widget;
+
+	widget = bacon_volume_button_new (GTK_ICON_SIZE_MENU,
+					  0, 100, -1);
+	gtk_widget_show (widget);
+
+	return widget;
+}
+
+GtkWidget *
+totem_statusbar_create (void)
+{
+	GtkWidget *widget;
+
+	widget = totem_statusbar_new ();
+	totem_statusbar_set_has_resize_grip (TOTEM_STATUSBAR (widget), FALSE);
+	if (arg_statusbar != FALSE)
+		gtk_widget_show (widget);
+
+	return widget;
+}
+
 static gboolean
 parse_xid (const gchar *option_name,
 	   const gchar *value,
@@ -844,12 +917,13 @@ static GOptionEntry option_entries [] =
 {
 	{ TOTEM_OPTION_WIDTH, 0, 0, G_OPTION_ARG_INT, &arg_width, NULL, NULL },
 	{ TOTEM_OPTION_HEIGHT, 0, 0, G_OPTION_ARG_INT, &arg_height, NULL, NULL },
-	{ TOTEM_OPTION_URL, 0, 0, G_OPTION_ARG_FILENAME /* FIXME: this should be STRING */, &arg_url, NULL, NULL },
+	{ TOTEM_OPTION_URL, 0, 0, G_OPTION_ARG_STRING, &arg_url, NULL, NULL },
 	{ TOTEM_OPTION_HREF, 0, 0, G_OPTION_ARG_STRING, &arg_href, NULL, NULL },
 	{ TOTEM_OPTION_XID, 0, 0, G_OPTION_ARG_CALLBACK, parse_xid, NULL, NULL },
 	{ TOTEM_OPTION_TARGET, 0, 0, G_OPTION_ARG_STRING, &arg_target, NULL, NULL },
 	{ TOTEM_OPTION_MIMETYPE, 0, 0, G_OPTION_ARG_STRING, &arg_mime_type, NULL, NULL },
 	{ TOTEM_OPTION_CONTROLS_HIDDEN, 0, 0, G_OPTION_ARG_NONE, &arg_no_controls, NULL, NULL },
+	{ TOTEM_OPTION_STATUSBAR, 0, 0, G_OPTION_ARG_NONE, &arg_statusbar, NULL, NULL },
 	{ TOTEM_OPTION_HIDDEN, 0, 0, G_OPTION_ARG_NONE, &arg_hidden, NULL, NULL },
 	{ TOTEM_OPTION_PLAYLIST, 0, 0, G_OPTION_ARG_NONE, &arg_is_playlist, NULL, NULL },
 	{ TOTEM_OPTION_REPEAT, 0, 0, G_OPTION_ARG_NONE, &arg_repeat, NULL, NULL },
@@ -935,7 +1009,11 @@ int main (int argc, char **argv)
 	emb->href = arg_href;
 	emb->filename = arg_remaining ? arg_remaining[0] : NULL;
 	emb->target = arg_target;
-	emb->mimetype = arg_mime_type;
+	if (arg_mime_type != NULL) {
+		emb->mimetype = arg_mime_type;
+	} else {
+		emb->mimetype = g_strdup (gnome_vfs_get_mime_type_for_name (emb->filename));
+	}
 	emb->hidden = arg_hidden;
 	emb->is_playlist = arg_is_playlist;
 	emb->repeat = arg_repeat;
