@@ -45,6 +45,7 @@
 
 #define MIN_LEN_FOR_SEEK 25000
 #define HALF_SECOND G_USEC_PER_SEC * .5
+#define BORING_IMAGE_VARIANCE 256.0		/* Tweak this if necessary */
 
 gboolean finished = FALSE;
 static char *format = "png";
@@ -207,16 +208,57 @@ add_holes_to_pixbuf_large (GdkPixbuf *pixbuf, int size)
 	return small;
 }
 
+/* This function attempts to detect images that are mostly solid images 
+ * It does this by calculating the statistical variance of the
+ * black-and-white image */
+static gboolean
+is_image_interesting (GdkPixbuf *pixbuf)
+{
+	/* We're gonna assume 8-bit samples. If anyone uses anything different,
+	 * it doesn't really matter cause it's gonna be ugly anyways */
+	int rowstride = gdk_pixbuf_get_rowstride(pixbuf);
+	int height = gdk_pixbuf_get_height(pixbuf);
+	guchar* buffer = gdk_pixbuf_get_pixels(pixbuf);
+	int num_samples = (rowstride * height);
+	int i;
+	float x_bar = 0.0f;
+	float variance = 0.0f;
+
+	/* FIXME: If this proves to be a performance issue, this function
+	 * can be modified to perhaps only check 3 rows. I doubt this'll
+	 * be a problem though. */
+
+	/* Iterate through the image to calculate x-bar */
+	for (i = 0; i < num_samples; i++) {
+		x_bar += (float)buffer[i];
+	}
+	x_bar /= ((float)num_samples);
+
+	/* Calculate the variance */
+	for (i = 0; i < num_samples; i++) {
+		float tmp = ((float)buffer[i] - x_bar);
+		variance += tmp * tmp;
+	}
+	variance /= ((float)(num_samples - 1));
+
+	return (variance > BORING_IMAGE_VARIANCE);
+}
+
 static void
 save_pixbuf (GdkPixbuf *pixbuf, const char *path,
-	     const char *video_path, int size)
+	     const char *video_path, int size, gboolean is_still)
 {
+	int width, height;
 	GdkPixbuf *small, *with_holes;
 	GError *err = NULL;
+	char *a_width, *a_height;
+
+	height = gdk_pixbuf_get_height (pixbuf);
+	width = gdk_pixbuf_get_width (pixbuf);
 
 	if (size <= 256)
 	{
-		int width, height, d_width, d_height;
+		int d_width, d_height;
 
 		height = gdk_pixbuf_get_height (pixbuf);
 		width = gdk_pixbuf_get_width (pixbuf);
@@ -233,17 +275,30 @@ save_pixbuf (GdkPixbuf *pixbuf, const char *path,
 		small = gdk_pixbuf_scale_simple (pixbuf, d_width, d_height,
 				GDK_INTERP_TILES);
 
-		with_holes = add_holes_to_pixbuf_small (small,
-				d_width, d_height);
-		g_return_if_fail (with_holes != NULL);
-		gdk_pixbuf_unref (small);
+		if (is_still == FALSE) {
+			with_holes = add_holes_to_pixbuf_small (small,
+					d_width, d_height);
+			g_return_if_fail (with_holes != NULL);
+			gdk_pixbuf_unref (small);
+		} else {
+			with_holes = small;
+		}
 	} else {
 		with_holes = add_holes_to_pixbuf_large (pixbuf, size);
 		g_return_if_fail (with_holes != NULL);
 	}
 
-	if (gdk_pixbuf_save (with_holes, path, format, &err, NULL) == FALSE)
+	a_width = g_strdup_printf ("%d", width);
+	a_height = g_strdup_printf ("%d", height);
+
+	if (gdk_pixbuf_save (with_holes, path, format, &err,
+				"tEXt::Thumb::Image::Width", a_width,
+				"tEXt::Thumb::Image::Height", a_height,
+				NULL) == FALSE)
 	{
+		g_free (a_width);
+		g_free (a_height);
+
 		if (err != NULL)
 		{
 			g_print ("totem-video-thumbnailer couln't write the thumbnail '%s' for video '%s': %s\n", path, video_path, err->message);
@@ -263,50 +318,6 @@ save_pixbuf (GdkPixbuf *pixbuf, const char *path,
 	gdk_pixbuf_unref (with_holes);
 }
 
-static void
-save_still_pixbuf (GdkPixbuf *pixbuf, const char *path,
-		const char *video_path, int size)
-{
-	GdkPixbuf *small;
-	int width, height, d_width, d_height;
-	GError *err = NULL;
-
-	height = gdk_pixbuf_get_height (pixbuf);
-	width = gdk_pixbuf_get_width (pixbuf);
-
-	if (width > height)
-	{
-		d_width = size;
-		d_height = size * height / width;
-	} else {
-		d_height = size;
-		d_width = size * width / height;
-	}
-
-	small = gdk_pixbuf_scale_simple (pixbuf, d_width, d_height,
-			GDK_INTERP_TILES);
-
-	if (gdk_pixbuf_save (small, path, "png", &err, NULL) == FALSE)
-	{
-		if (err != NULL)
-		{
-			g_print ("totem-video-thumbnailer couln't write the thumbnail '%s' for video '%s': %s\n", path, video_path, err->message);
-			g_error_free (err);
-		} else {
-			g_print ("totem-video-thumbnailer couln't write the thumbnail '%s' for video '%s'\n", path, video_path);
-		}
-
-		gdk_pixbuf_unref (small);
-		return;
-	}
-
-#ifdef THUMB_DEBUG
-	show_pixbuf (small);
-#endif
-
-	gdk_pixbuf_unref (small);
-}
-
 static gpointer
 time_monitor (gpointer data)
 {
@@ -327,6 +338,15 @@ usage (const char *cmd)
 	g_print ("Usage: %s [-s <size>] [-j] <input> <output> [backend options]\n", cmd);
 }
 
+static gboolean
+video_is_still_image (BaconVideoWidget *bvw)
+{
+	GValue value = { 0, };
+
+	bacon_video_widget_get_metadata (bvw, BVW_INFO_STILL_IMAGE, &value);
+	return g_value_get_boolean (&value);
+}
+
 int main (int argc, char *argv[])
 {
 	GError *err = NULL;
@@ -335,13 +355,20 @@ int main (int argc, char *argv[])
 	int i, length, size;
 	char *input, *output;
 
+	const float frame_locations[] =
+	{ 1.0 / 3.0, 2.0 / 3.0, 0.1,0.9, 0.5 };
+	const int frame_locations_length = 5;
+	int current;
+
 	if (argc <= 2 || strcmp (argv[1], "-h") == 0 ||
 	    strcmp (argv[1], "--help") == 0) {
 		usage (argv[0]);
 		return -1;
 	}
 
+#ifdef G_OS_UNIX
 	nice (20);
+#endif
 
 	g_thread_init (NULL);
 #ifndef THUMB_DEBUG
@@ -394,24 +421,6 @@ int main (int argc, char *argv[])
 
 	if (bacon_video_widget_open (bvw, input, &err) == FALSE)
 	{
-		if (err->code == BVW_ERROR_STILL_IMAGE)
-		{
-			g_error_free (err);
-			err = NULL;
-			pixbuf = gdk_pixbuf_new_from_file (input, &err);
-			if (pixbuf != NULL)
-			{
-				save_still_pixbuf (pixbuf,
-						output, input, size);
-				gdk_pixbuf_unref (pixbuf);
-				exit (0);
-			}
-			g_print ("totem-video-thumbnailer couln't open file '%s'\n"
-					"Reason: %s.\n", input, err->message);
-			g_error_free (err);
-			exit (1);
-		}
-
 		g_print ("totem-video-thumbnailer couln't open file '%s'\n"
 				"Reason: %s.\n",
 				input, err->message);
@@ -419,6 +428,20 @@ int main (int argc, char *argv[])
 		exit (1);
 	}
 
+	if (video_is_still_image (bvw) != FALSE) {
+		pixbuf = gdk_pixbuf_new_from_file (input, &err);
+		if (pixbuf != NULL)
+		{
+			save_pixbuf (pixbuf, output, input, size, TRUE);
+			gdk_pixbuf_unref (pixbuf);
+			exit (0);
+		}
+		g_print ("totem-video-thumbnailer couln't open file '%s'\n"
+				"Reason: %s.\n", input, err->message);
+		g_error_free (err);
+		exit (1);
+	}
+
 	bacon_video_widget_play (bvw, &err);
 	if (err != NULL)
 	{
@@ -429,31 +452,43 @@ int main (int argc, char *argv[])
 		exit (1);
 	}
 
-	/* Jump a 3rd into the file */
-	length = bacon_video_widget_get_stream_length (bvw);
-	if (length > MIN_LEN_FOR_SEEK)
+	/* Test at multiple points in the file to see if we can get an 
+	 * interesting frame */
+	for (current = 0; current < frame_locations_length; current++)
 	{
-		if (bacon_video_widget_seek
-				(bvw, ((float) 1 / 3), NULL) == FALSE)
+		length = bacon_video_widget_get_stream_length (bvw);
+		if (length > MIN_LEN_FOR_SEEK)
 		{
-			bacon_video_widget_play (bvw, NULL);
+			if (bacon_video_widget_seek
+					(bvw, frame_locations[current], NULL) == FALSE)
+			{
+				bacon_video_widget_play (bvw, NULL);
+			}
 		}
+
+		if (bacon_video_widget_can_get_frames (bvw, &err) == FALSE)
+		{
+			g_print ("totem-video-thumbnailer: '%s' isn't thumbnailable\n"
+					"Reason: %s\n",
+					input, err ? err->message : "programming error");
+			bacon_video_widget_close (bvw);
+			gtk_widget_destroy (GTK_WIDGET (bvw));
+			g_error_free (err);
+
+			exit (1);
+		}
+
+		/* Pull the frame, if it's interesting we bail early */
+		pixbuf = bacon_video_widget_get_current_frame (bvw);
+		if (is_image_interesting (pixbuf) != FALSE)
+			break;
+
+		/* If we get to the end of this loop, we'll end up using the last image
+		 * we pulled */
+		if(current + 1 < frame_locations_length)
+			gdk_pixbuf_unref (pixbuf);
 	}
 
-	if (bacon_video_widget_can_get_frames (bvw, &err) == FALSE)
-	{
-		g_print ("totem-video-thumbnailer: '%s' isn't thumbnailable\n"
-				"Reason: %s\n",
-				input, err ? err->message : "programming error");
-		bacon_video_widget_close (bvw);
-		gtk_widget_destroy (GTK_WIDGET (bvw));
-		g_error_free (err);
-
-		exit (1);
-	}
-
-	pixbuf = bacon_video_widget_get_current_frame (bvw);
-
 	/* Cleanup */
 	bacon_video_widget_close (bvw);
 	finished = TRUE;
@@ -466,7 +501,7 @@ int main (int argc, char *argv[])
 		exit (1);
 	}
 
-	save_pixbuf (pixbuf, output, input, size);
+	save_pixbuf (pixbuf, output, input, size, FALSE);
 	gdk_pixbuf_unref (pixbuf);
 
 	return 0;
