@@ -258,7 +258,7 @@ on_about1_activate (GtkButton *button, Vanity *vanity)
 
 	vanity->about = g_object_new (GTK_TYPE_ABOUT_DIALOG,
 			"name", _("Vanity"),
-			"version", PACKAGE_VERSION,
+			"version", VERSION,
 			"copyright", _("Copyright \xc2\xa9 2002-2005 Bastien Nocera"),
 			"comments", description,
 			"authors", authors,
@@ -655,7 +655,7 @@ main (int argc, char **argv)
 
 	gtk_init (&argc, &argv);
 #ifndef HAVE_GTK_ONLY
-	gnome_program_init ("vanity", PACKAGE_VERSION,
+	gnome_program_init ("vanity", VERSION,
 			LIBGNOMEUI_MODULE,
 			argc, argv,
 			GNOME_PARAM_APP_DATADIR, DATADIR,
@@ -682,9 +682,11 @@ main (int argc, char **argv)
 	gnome_authentication_manager_init ();
 #endif /* !HAVE_GTK_ONLY */
 
+#ifdef TOTEM_RUN_IN_SOURCE_TREE
 	if (g_file_test ("../data/vanity.glade", G_FILE_TEST_EXISTS) != FALSE)
 		filename = g_strdup ("../data/vanity.glade");
 	else
+#endif
 		filename = g_build_filename (DATADIR,
 				"totem", "vanity.glade", NULL);
 
